<?php
/*
Plugin Name: WooCommerce Pei Payment Gateway
Plugin URI: http://www.premis.is
Description: Pei Payment gateway for Woocommerce 3.x
Version: 3.02
Author: Premis
Author URI: http://www.premis.is
*/


add_action('plugins_loaded', 'init_pei_gateway', 0);

function init_pei_gateway()
{
    if(!class_exists('WC_Payment_Gateway')) return;

    class WC_Pei_Gateway extends WC_Payment_Gateway
    {
        public function __construct()
        {
            /*
              Set basic info for payment gateway
            */
            $this->id                 = 'dg_pei';
            $this->icon               = plugins_url('/pei_logo_small.png', __FILE__);
            $this->has_fields         = true;
            $this->method_title       = 'Pei';
            $this->method_description = 'Borga með Pei';

            /*
              Create form fields and setting fileds
            */
            $this->init_form_fields();
            $this->init_settings();

            /*
              Set the corrects urls for testing and production
            */
            $this->auth_url    = $this->get_option('test') == 'yes' ? 'https://authstaging.pei.is/core/connect/token' : 'https://auth.pei.is/core/connect/token';
            $this->order_url   = $this->get_option('test') == 'yes' ? 'https://externalapistaging.pei.is/api/orders' : 'https://api.pei.is/api/orders';
            $this->payment_url = $this->get_option('test') == 'yes' ? 'https://gattinstaging.pei.is/' : 'https://gattin.pei.is/';

            /*
              Get all the needed settings from user
              These settings are set in the Wordpress admin interface
            */
            $this->title            = $this->get_option('title');
            $this->merchantId       = $this->get_option('merchantId');
            $this->clientId         = $this->get_option('user');
            $this->secret           = $this->get_option('secret');
            $this->redirect_page_id = $this->get_option('redirect_page_id');
            $this->description      = $this->get_option('description');
            $this->logging          = $this->get_option('logging');
            $this->test             = $this->get_option('test');


            /*
              Hook into callback api
            */
            add_action('woocommerce_api_'.strtolower(get_class($this)), array(&$this, 'handle_callback'));

            /*
              Hook into the thankyou page, and call check_pei_response when hook is activated
              You are redirected to the thankyou page by pei.is
            */
            // if (WC()->version < 2.6) {
            //     add_action('woocommerce_thankyou_dg_pei', array(
            //         &$this,
            //         'check_pei_response'
            //     ));
            // }

            /*
              Hook into update options, and call function process_admin_options when hook is activated
            */
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(
                $this,
                'process_admin_options'
            ));
        }


        /*
          Create options in admin interface
        */
        function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                    'title' => __('Enable/Disable', 'dg_pei'),
                    'type' => 'checkbox',
                    'label' => __('Enable Pei', 'dg_pei'),
                    'default' => 'yes'
                ),
                'title' => array(
                    'title' => __('Title', 'dg_pei'),
                    'type' => 'text',
                    'description' => __('This controls the title which the user sees during checkout.', 'dg_pei'),
                    'default' => __('Pei', 'dg_pei'),
                    'desc_tip' => true
                ),
                'description' => array(
                    'title' => __('Customer Message', 'dg_pei'),
                    'type' => 'textarea',
                    'deafult' => ''
                ),
                'user' => array(
                    'title' => __('Pei User', 'dg_pei'),
                    'type' => 'text',
                    'default' => 'democlient',
                    'description' => __('Username')
                ),
                'secret' => array(
                    'title' => __('Pei Secret', 'dg_pei'),
                    'type' => 'textarea',
                    'default' => 'demosecret',
                    'description' => __('Secret, finnur þetta inná https://pei.is accountinu þínu')
                ),
                'merchantId' => array(
                    'title' => __('Merchant ID', 'dg_pei'),
                    'type' => 'text',
                    'default' => '1',
                    'description' => __('Finnur þetta inná https://pei.is accountinu þínu')
                ),
                'test' => array(
                    'title' => __('Prófunarumhverfi', 'dg_pei'),
                    'type' => 'checkbox',
                    'label' => __('Senda á prófunarumhverfi Pei', 'dg_pei'),
                    'default' => 'yes'
                ),
                /*'logging' => array(
                    'title' => __('Logging', 'dg_pei'),
                    'type' => 'checkbox',
                    'label' => __('Enable Logging', 'dg_pei'),
                    'default' => 'no',
                    'description' => __('Það er hægt að finna logginn inná servernum undir wp-content/debug.log. Ekki hafa kveikt á þessu nema að það sé brýn nauðsyn')
                ),*/
            );
        }

        /*
          Check pei response
        */
        function handle_callback() {
            error_log("--- post ---");
            $data = json_decode(file_get_contents('php://input'), true);
            error_log("echoing get");
            error_log(var_export($_GET, true));
            //$order_id = $_GET['order_id'];
            $token = $this->authorize();

            $args = array(
                'headers' => array(
                    'Authorization' => $token,
                    'content-type' => "application/json"
                ),
            );

            $response = wp_remote_get($this->order_url . "/" . $data['orderId'], $args);
            error_log("response from callback callback");
            error_log(var_export($response, true));

            if (!is_array($response)) {
                $order->add_order_note("Ekkert svar frá Pei");
                return false;
            } else {
                $response = json_decode($response['body']);

                if ($response->isConfirmed) {
                    global $woocommerce;
                    $order = wc_get_order($response->reference);
                    $order->payment_complete();
                    $order->add_order_note('Búið að borga með Pei');
                }
            }

            // wp_die();
        }

        /*
      Returns auth tokens for Pei
    */
    function authorize() {
        $data = array(
            'grant_type' => 'client_credentials',
            'scope' => 'externalapi'
        );

        // Create the payload, the payload consists of the username and secret that the vendor sets
        $auth_payload = $this->test == 'yes' ? base64_encode('democlient:demosecret') : base64_encode("{$this->clientId}:{$this->secret}");

        // Create the http parameters with the payload and data included
        $args = array(
            'headers' => array(
                'Authorization' => 'Basic ' . $auth_payload
            ),
            'body' => http_build_query($data)
        );

        // Send the post request and log
        $response = wp_remote_post($this->auth_url, $args);
        error_log(var_export($response, true));

        // If there is no response
        if (!is_array($response)) {
            return false;
        } else {
            // Create an array from the json response from server
            $response = json_decode($response['body']);
            // Move onto the order stage
            return $response->token_type . ' ' . $response->access_token;
        }
    }

        /*
          Show description and add a field for the buyers ssn, the ssn is needed for phonenumber checks at pei.is
        */
        function payment_fields()
        {
            if ($this->description)
                echo wpautop(wptexturize($this->description));
            ?>
            <input type="text" name="dg_pei-ssn" placeholder="Kennitala"/>
            <?php
        }

        /*
          Gets called when the user gets to the thankyou page.
          The user is redirected to the thankyou page by pei.is
        */
        function check_pei_response()
        {
            // Get the order from the order id
            global $woocommerce;
            $order = new WC_Order($_GET['order-received']);

            // Set the payment as complete
            $order->payment_complete();
            // Add a note
            $order->add_order_note('Búið að borga með Pei');
            // Empty cart... Should be a part of payment_complete
            $woocommerce->cart->empty_cart();
        }

        /*
          Gets called when the user presses the buy button
        */
        function process_payment($order_id)
        {
            // Get order information
            global $woocommerce;
            $order = new WC_Order($order_id);

            // Get auth token from pei
            $token = $this->authorize();

            // If there was no respone redirect to checkout and add a note to the order
            if ($token == false) {
                error_log("no token");
                error_log(var_export($response, true));
                $order->add_order_note("Eitthvað vesen með að fá svar frá pei");
                return array(
                    'result' => 'failed',
                    'redirect' => $woocommerce->cart->get_checkout_url()
                );
            } else {
                // Move onto the order stage
                return $this->Order($token, $order);
            }
        }

        /*
          Parameters:
          $token: Bearer token provided by pei.is auth service
          $order: $order provided by WooCommerce

          Redirects to user to pei.is with all the relevant information
        */
        function Order($token, $order)
        {
            global $woocommerce;

            $postback_url = add_query_arg(
                array(
                    'wc-api' => 'WC_Pei_Gateway',
                    'order_id'=> $order->get_id()
                ), home_url( '/' )
            );

            error_log("Postback_url");
            error_log(var_export($postback_url, true));

            // Create the payload
            $fields = array(
                'merchantId' => $this->test == 'yes' ? 1 : $this->merchantId,
                'amount' => $order->get_total(),
                // Redirect here if it is a success
                'successReturnUrl' => $this->get_return_url($order),
                // Redirect here if the user cancels
                'cancelReturnUrl' => $woocommerce->cart->get_checkout_url(),
                // Url that Pei postbacks to
                'postbackUrl' => $postback_url,
                'reference' => $order->get_id(),
                'buyer' => array(
                    'name' => "{$order->get_billing_first_name()} {$order->get_billing_last_name()}",
                    'email' => $order->get_billing_email(),
                    'mobileNumber' => $order->get_billing_phone(),
                    'ssn' => str_replace(array(
                        ' ',
                        '-'
                    ), '', $_POST['dg_pei-ssn'])
                )
            );

            // Get all the items in order
            $items = $order->get_items();

            // Add items to payload
            foreach ($items as $item) {
                $productmeta = new WC_Product($item['product_id']);
                $unitPrice = $productmeta->get_price();
                $quantity = $item['qty'];
                $amount = $unitPrice * $quantity;


                $fields['items'][] = array(
                    'code' => $productmeta->get_sku(),
                    'name' => $item['name'],
                    'quantity' => $quantity,
                    'amount' => $amount,
                    'unitPrice' => $unitPrice
                );
            }

            error_log("token inside Order");
            error_log($token);

            // Create http request with the payload and auth token
            $args = array(
                'headers' => array(
                    "Authorization" => $token
                ),
                'body' => $fields
            );

            $response = wp_remote_post($this->order_url, $args);
            error_log("Pei response inside Order");
            error_log(var_export($response, true));


            // If the request failed redirect user to checkout and add note
            if (!is_array($response)) {
                error_log("Order request failed");
                error_log(var_export($response, true));
                $order->add_order_note("Pei neitaði order id-inu");
                return array(
                    'result' => 'failed',
                    'redirect' => $woocommerce->cart->get_checkout_url()
                );
            } else {
                // Add note that the user is at pei.is
                $order->add_order_note("Bíða eftir að einstaklingur klári að borga");
                // Puts the json response into a PHP array
                $response = json_decode($response['body']);

                error_log("User redirected to pei");
                error_log(var_export($response, true));
                // redirect user to the payment url
                return array(
                    'result' => 'success',
                    'redirect' => $this->payment_url . $response->orderId
                );
            }
        }
    }

    // Adds the gateway to WooCommerce
    function add_pei_gateway($methods)
{
    $methods[] = 'WC_Pei_Gateway';
    return $methods;
}

    add_filter('woocommerce_payment_gateways', 'add_pei_gateway');
}
?>
